/*
 * nodoka_draw.c
 * This file is part of gtk-nodoka-engine
 *
 * Copyright (C) 2007, 2008 - Martin Sourada, Daniel Geiger
 *
 * gtk-nodoka-engine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * gtk-nodoka-engine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gtk-nodoka-engine; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */ 


#include "nodoka_draw.h"
#include "nodoka_style.h"
#include "nodoka_types.h"

#include "support.h"

#include <cairo.h>

#define M_PI 3.14159265358979323846
#define GLASS_HILIGHT 1.1
#define BULGING_TOP_HILIGHT 1.105
#define BULGING_BOTTOM_HILIGHT 1.06
#define HOLLOW_TOP_HILIGHT 1.147
#define HOLLOW_BOTTOM_HILIGHT 1
#define DARK_TOP_HILIGHT 1.487
#define DARK_BOTTOM_HILIGHT 1
#define GRADIENT_CENTER 0.7

// Static Nodoka Functions
static void
nodoka_rounded_rectangle_fast (cairo_t * cr, double x, double y, double w,
							   double h, uint8 corners)
{
	const float RADIUS_CORNERS = 0.36;

	if (corners & NDK_CORNER_TOPLEFT)
		cairo_move_to (cr, x + RADIUS_CORNERS, y);
	else
		cairo_move_to (cr, x, y);

	if (corners & NDK_CORNER_TOPRIGHT)
	{
		cairo_line_to (cr, x + w - RADIUS_CORNERS, y);
		cairo_move_to (cr, w + x, y + RADIUS_CORNERS);
	}
	else
		cairo_line_to (cr, x + w, y);

	if (corners & NDK_CORNER_BOTTOMRIGHT)
	{
		cairo_line_to (cr, w + x, h + y - RADIUS_CORNERS);
		cairo_move_to (cr, w + x - RADIUS_CORNERS, h + y);
	}
	else
		cairo_line_to (cr, w + x, h + y);

	if (corners & NDK_CORNER_BOTTOMLEFT)
	{
		cairo_line_to (cr, x + RADIUS_CORNERS, h + y);
		cairo_move_to (cr, x, h + y - RADIUS_CORNERS);
	}
	else
		cairo_line_to (cr, x, h + y);

	if (corners & NDK_CORNER_TOPLEFT)
		cairo_line_to (cr, x, y + RADIUS_CORNERS);
	else
	{
		if (corners == NDK_CORNER_NONE)
			cairo_close_path (cr);
		else
			cairo_line_to (cr, x, y);
	}
}

static void
clearlooks_rounded_rectangle (cairo_t * cr,
							  double x, double y, double w, double h,
							  int radius, uint8 corners)
{
	if (corners & NDK_CORNER_TOPLEFT)
		cairo_move_to (cr, x + radius, y);
	else
		cairo_move_to (cr, x, y);

	if (corners & NDK_CORNER_TOPRIGHT)
		cairo_arc (cr, x + w - radius, y + radius, radius, M_PI * 1.5,
				   M_PI * 2);
	else
		cairo_line_to (cr, x + w, y);

	if (corners & NDK_CORNER_BOTTOMRIGHT)
		cairo_arc (cr, x + w - radius, y + h - radius, radius, 0, M_PI * 0.5);
	else
		cairo_line_to (cr, x + w, y + h);

	if (corners & NDK_CORNER_BOTTOMLEFT)
		cairo_arc (cr, x + radius, y + h - radius, radius, M_PI * 0.5, M_PI);
	else
		cairo_line_to (cr, x, y + h);

	if (corners & NDK_CORNER_TOPLEFT)
		cairo_arc (cr, x + radius, y + radius, radius, M_PI, M_PI * 1.5);
	else
		cairo_line_to (cr, x, y);
}

static void
nodoka_rounded_rectangle (cairo_t * cr,
						  double x, double y, double w, double h,
						  int radius, uint8 corners)
{
	radius < 2 ? nodoka_rounded_rectangle_fast (cr, x, y, w, h,
												corners) :
		clearlooks_rounded_rectangle (cr, x, y, w, h, radius, corners);
}

static void
rotate_mirror_translate (cairo_t * cr, double radius, double x, double y,
						 boolean mirror_horizontally, boolean mirror_vertically)
{
	cairo_matrix_t matrix_rotate;
	cairo_matrix_t matrix_mirror;
	cairo_matrix_t matrix_result;

	double r_cos = cos (radius);
	double r_sin = sin (radius);

	cairo_matrix_init (&matrix_rotate, r_cos, r_sin, r_sin, r_cos, x, y);

	cairo_matrix_init (&matrix_mirror, mirror_horizontally ? -1 : 1, 0, 0,
					   mirror_vertically ? -1 : 1, 0, 0);

	cairo_matrix_multiply (&matrix_result, &matrix_mirror, &matrix_rotate);

	cairo_set_matrix (cr, &matrix_result);
}

static void
nodoka_set_grad_full (cairo_t * cr, const NodokaRGB * color, double top_hilight,
					  double bottom_hilight, double gradient_center, int width,
					  int height, boolean gradients, boolean transparent,
					  double alpha)
{
	if (gradients)
	{
		cairo_pattern_t *pattern;

		NodokaRGB top_shade;
		NodokaRGB bottom_shade;

		nodoka_shade (color, &top_shade, top_hilight);
		nodoka_shade (color, &bottom_shade, bottom_hilight);

		pattern = cairo_pattern_create_linear (0, 0, width, height);
		cairo_pattern_add_color_stop_rgb (pattern, 0, top_shade.r, top_shade.g,
										  top_shade.b);
		cairo_pattern_add_color_stop_rgb (pattern, gradient_center, color->r,
										  color->g, color->b);
		cairo_pattern_add_color_stop_rgb (pattern, 1, bottom_shade.r,
										  bottom_shade.g, bottom_shade.b);

		cairo_set_source (cr, pattern);
		cairo_pattern_destroy (pattern);
	}
	else
	{
		if (transparent)
			cairo_set_source_rgba (cr, color->r, color->g, color->b, alpha);
		else
			cairo_set_source_rgb (cr, color->r, color->g, color->b);
	}
}

static void
nodoka_set_gradient (cairo_t * cr, const NodokaRGB * color, double hilight,
					 int width, int height, boolean gradients,
					 boolean transparent)
{
	nodoka_set_grad_full (cr, color, hilight, hilight, 0.5, width, height,
						  gradients, transparent, 0.8);
}

//Widget Functions
void
nodoka_draw_button (cairo_t * cr,
					const NodokaColors * colors,
					const WidgetParameters * widget,
					int x, int y, int width, int height, boolean horizontal,
					boolean shadows)
{
	double xoffset = 0, yoffset = 0;
	NodokaRGB *fill = (NodokaRGB *) & colors->bg[widget->state_type];
	NodokaRGB border_disabled = colors->shade[4];
	NodokaRGB border_normal;
	nodoka_shade (&colors->shade[6], &border_normal, 0.95);
	NodokaRGB hilight;
	nodoka_shade (fill, &hilight, 1.1 * widget->hilight_ratio);
	double alpha = 1;

	cairo_translate (cr, x, y);
	cairo_set_line_width (cr, 1.0);

	if (widget->xthickness == 3)
		xoffset = 1;
	if (widget->ythickness == 3)
		yoffset = 1;

	nodoka_rounded_rectangle (cr, xoffset, yoffset, width - (xoffset * 2),
							  height - (yoffset * 2), widget->roundness,
							  widget->corners);
	if (widget->disabled)
	{
		cairo_set_source_rgba (cr, border_disabled.r, border_disabled.g,
							   border_disabled.b, 0.15);
		cairo_stroke (cr);
		border_disabled.r = border_disabled.r * (0.6) + fill->r * 0.4;
		border_disabled.g = border_disabled.g * (0.6) + fill->g * 0.4;
		border_disabled.b = border_disabled.b * (0.6) + fill->b * 0.4;
		alpha = 0.35;
	}
	else
	{
		cairo_set_source_rgba (cr, border_normal.r, border_normal.g,
							   border_normal.b, 0.15);
		cairo_stroke (cr);
		border_normal.r = border_normal.r * (0.6) + fill->r * 0.4;
		border_normal.g = border_normal.g * (0.6) + fill->g * 0.4;
		border_normal.b = border_normal.b * (0.6) + fill->b * 0.4;
	}

	//bg
	if (widget->roundness < 2)
		cairo_rectangle (cr, xoffset + 1, yoffset + 1,
						 width - (xoffset * 2) - 2, height - (yoffset * 2) - 2);
	else
		clearlooks_rounded_rectangle (cr, xoffset + 0.5, yoffset + 0.5,
									  width - (xoffset * 2) - 1,
									  height - (yoffset * 2) - 1,
									  widget->roundness, widget->corners);

	nodoka_set_grad_full (cr, fill, (widget->active ? HOLLOW_TOP_HILIGHT
									 : BULGING_TOP_HILIGHT),
						  (widget->active ? HOLLOW_BOTTOM_HILIGHT
						   : BULGING_BOTTOM_HILIGHT),
						  GRADIENT_CENTER, (horizontal ? 0 : width),
						  (horizontal ? height : 0), widget->gradients,
						  FALSE, alpha);

	cairo_save (cr);
	if (widget->roundness > 1)
		cairo_clip_preserve (cr);

	int curve_pos = 1;
	if (widget->roundness < 2)
		curve_pos = 2;

	cairo_fill (cr);
	nodoka_shade (fill, &hilight, 1.08 * widget->hilight_ratio);
	cairo_restore (cr);

	//button shadow
	if (!(widget->disabled) && shadows)
	{
		cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 0.1);
		if (widget->active)
			nodoka_rounded_rectangle (cr, xoffset + 1.5, yoffset + 1.5,
									  width - (xoffset * 2) - 2,
									  height - (yoffset * 2) - 2,
									  widget->roundness - 1, widget->corners);
		else
			nodoka_rounded_rectangle (cr, xoffset + 0.5, yoffset + 0.5,
									  width - (xoffset * 2),
									  height - (yoffset * 2),
									  widget->roundness + 1, widget->corners);
		cairo_stroke (cr);
	}

	//border
	if (widget->disabled)
		cairo_set_source_rgb (cr, border_disabled.r, border_disabled.g,
							  border_disabled.b);
	else
		cairo_set_source_rgb (cr, border_normal.r, border_normal.g,
							  border_normal.b);
	nodoka_rounded_rectangle (cr, xoffset + 0.5, yoffset + 0.5,
							  width - (xoffset * 2) - 1,
							  height - (yoffset * 2) - 1, widget->roundness,
							  widget->corners);
	cairo_stroke (cr);


}

void
nodoka_draw_entry (cairo_t * cr,
				   const NodokaColors * colors,
				   const WidgetParameters * widget,
				   int x, int y, int width, int height)
{
	NodokaRGB *base = (NodokaRGB *) & colors->base[widget->state_type];
	NodokaRGB *border;

	if (widget->focus)
		border = (NodokaRGB *) & colors->spot[2];
	else
		border = (NodokaRGB *) & colors->shade[widget->disabled ? 3 : 5];

	cairo_translate (cr, x + 0.5, y + 0.5);
	cairo_set_line_width (cr, 1.0);

	/* Fill the background (shouldn't have to) */
	cairo_rectangle (cr, -0.5, -0.5, width, height);
	cairo_set_source_rgb (cr, widget->parentbg.r, widget->parentbg.g,
						  widget->parentbg.b);
	cairo_fill (cr);

	/* Fill the entry's base color (why isn't is large enough by default?) */
	cairo_rectangle (cr, 1.5, 1.5, width - 4, height - 4);
	cairo_set_source_rgb (cr, base->r, base->g, base->b);
	cairo_fill (cr);

	/* Draw the border */
	cairo_set_source_rgb (cr, border->r, border->g, border->b);
	if (widget->roundness < 2)
		cairo_rectangle (cr, 1, 1, width - 3, height - 3);
	else
		clearlooks_rounded_rectangle (cr, 1, 1, width - 3, height - 3,
									  (widget->roundness == 2 ? 2 : 3),
									  widget->corners);
	cairo_stroke (cr);

	/* Draw the focused border */
	if (widget->focus)
	{
		if (widget->roundness < 2)
			cairo_rectangle (cr, 2, 2, width - 5, height - 5);
		else
			clearlooks_rounded_rectangle (cr, 2, 2, width - 5, height - 5, 2,
										  widget->corners);
		cairo_set_source_rgb (cr, colors->spot[1].r, colors->spot[1].g,
							  colors->spot[1].b);
		cairo_stroke (cr);
	}
	else
	{
		cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, widget->disabled ? 0 : 0.15);

		cairo_move_to (cr, 2, height - 3);	//2
		cairo_line_to (cr, 2, 2);	//3
		cairo_line_to (cr, 2, 2);	//3
		cairo_line_to (cr, width - 3, 2);	//2
		cairo_stroke (cr);
	}

}

void
nodoka_draw_spinbutton_down (cairo_t * cr,
							 const NodokaColors * colors,
							 const WidgetParameters * widget,
							 int x, int y, int width, int height)
{
	NodokaRGB shadow;
	nodoka_shade (&colors->bg[0], &shadow, 0.8);

	cairo_pattern_t *pattern;

	cairo_translate (cr, x + 1, y + 1);

	cairo_rectangle (cr, 1, 1, width - 4, height - 4);

	pattern = cairo_pattern_create_linear (0, 0, 0, height);
	cairo_pattern_add_color_stop_rgb (pattern, 0.0, shadow.r, shadow.g,
									  shadow.b);
	cairo_pattern_add_color_stop_rgba (pattern, 1.0, shadow.r, shadow.g,
									   shadow.b, 0.0);

	cairo_set_source (cr, pattern);
	cairo_fill (cr);

	cairo_pattern_destroy (pattern);
}

static void
nodoka_scale_draw_gradient (cairo_t * cr,
							const NodokaRGB * c1,
							const NodokaRGB * c2,
							int x, int y, int width, int height,
							boolean alpha, boolean horizontal)
{
	if (alpha)
		cairo_set_source_rgba (cr, c1->r, c1->g, c1->b, 0.4);
	else
		cairo_set_source_rgb (cr, c1->r, c1->g, c1->b);

//draw gradient for bg
	nodoka_set_grad_full (cr, c1, HOLLOW_BOTTOM_HILIGHT, HOLLOW_TOP_HILIGHT,
						  GRADIENT_CENTER, (horizontal ? 0 : width),
						  (horizontal ? height : 0), TRUE, FALSE, 1.0);

	cairo_rectangle (cr, x, y, width, height);
	cairo_fill (cr);

	cairo_rectangle (cr, x, y, width, height);
	cairo_set_source_rgba (cr, c2->r, c2->g, c2->b, 0.8);
	cairo_stroke (cr);
}

void
nodoka_draw_scale_trough (cairo_t * cr,
						  const NodokaColors * colors,
						  const WidgetParameters * widget,
						  const SliderParameters * slider,
						  int x, int y, int width, int height)
{
	int fill_x, fill_y, fill_width, fill_height;	/* Fill x,y,w,h     */
	int trough_width, trough_height;
	double translate_x, translate_y;
	int fill_size = slider->fill_size;
	int TROUGH_SIZE = 6;

	if (slider->horizontal)
	{
		if (fill_size > width - 3)
			fill_size = width - 3;

		fill_x = slider->inverted ? width - fill_size - 3 : 0;
		fill_y = 0;
		fill_width = fill_size;
		fill_height = TROUGH_SIZE - 2;

		trough_width = width - 3;
		trough_height = TROUGH_SIZE - 2;

		translate_x = x + 0.5;
		translate_y = y + 0.5 + (height / 2) - (TROUGH_SIZE / 2);
	}
	else
	{
		if (fill_size > height - 3)
			fill_size = height - 3;

		fill_x = 0;
		fill_y = slider->inverted ? height - fill_size - 3 : 0;
		fill_width = TROUGH_SIZE - 2;
		fill_height = fill_size;

		trough_width = TROUGH_SIZE - 2;
		trough_height = height - 3;

		translate_x = x + 0.5 + (width / 2) - (TROUGH_SIZE / 2);
		translate_y = y + 0.5;
	}

	cairo_set_line_width (cr, 1.0);
	cairo_translate (cr, translate_x, translate_y);

	cairo_translate (cr, 1, 1);

	nodoka_scale_draw_gradient (cr, &colors->shade[3],	//1
								&colors->shade[5],	//3 /* border */
								0, 0, trough_width, trough_height,
								TRUE, slider->horizontal);

//we want to draw the gradient using a direct gradient call
	nodoka_set_grad_full (cr, &colors->spot[1], DARK_BOTTOM_HILIGHT,
						  DARK_TOP_HILIGHT, GRADIENT_CENTER,
						  (slider->horizontal ? 0 : fill_width),
						  (slider->horizontal ? fill_height : 0), TRUE, FALSE,
						  1.0);
	cairo_rectangle (cr, fill_x + 0.5, fill_y + 0.5, fill_width - 1,
					 fill_height - 1);
	cairo_fill (cr);
//set the right border color
	cairo_rectangle (cr, fill_x, fill_y, fill_width, fill_height);
	cairo_set_source_rgba (cr, colors->spot[2].r, colors->spot[2].g,
						   colors->spot[2].b, 0.8);
	cairo_stroke (cr);

	/* Draw shadow */
	cairo_move_to (cr, 1, trough_height - 2);
	cairo_line_to (cr, 1, 1);
	cairo_line_to (cr, trough_width - 2, 1);
	cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 0.1);
	cairo_stroke (cr);
}

void
nodoka_draw_scale_slider (cairo_t * cr,
						  const NodokaColors * colors,
						  const WidgetParameters * widget,
						  int x, int y, int width, int height,
						  boolean horizontal)
{
	//draw button
	nodoka_draw_button (cr, colors, widget, x, y, width, height, horizontal,
						TRUE);

	//setting colors
	NodokaRGB fill;
	NodokaRGB fill2;

	fill = colors->bg[0];
	fill2 = colors->bg[widget->state_type];

	if (widget->prelight)
		nodoka_shade (&fill2, &fill, 1.06);

	NodokaRGB style;

	nodoka_shade (&fill, &style, 0.55);

	//rotate if not horizontal  
	if (!horizontal)
	{
		int tmp = height;

		rotate_mirror_translate (cr, M_PI / 2, x, y, FALSE, FALSE);
		height = width;
		width = tmp;
	}
	//draw scale slider decoration
	int circ_radius = 1;
	int circ_space = 5;
	int i;

	int circ_y = height / 2.0 - 2.5;
	int circ_x = width / 2.0 - (circ_space / 2.0);

	cairo_translate (cr, 0.5, 0.5);

	for (i = 0; i < 2; i++)
	{
		cairo_move_to (cr, circ_x, circ_y);

		cairo_arc (cr, circ_x, circ_y, circ_radius + 0.5, 0, M_PI * 2);
		cairo_close_path (cr);

		cairo_arc (cr, circ_x, circ_y + circ_space, circ_radius + 0.5, 0,
				   M_PI * 2);
		cairo_close_path (cr);

		cairo_set_source_rgba (cr, style.r, style.g, style.b, 0.2);
		cairo_fill (cr);

		cairo_arc (cr, circ_x, circ_y, circ_radius, 0, M_PI * 2);
		cairo_close_path (cr);

		cairo_arc (cr, circ_x, circ_y + circ_space, circ_radius, 0, M_PI * 2);
		cairo_close_path (cr);

		cairo_set_source_rgba (cr, style.r, style.g, style.b, 0.1);
		cairo_fill (cr);
		circ_x += circ_space;
	}
}

void
nodoka_draw_progressbar_trough (cairo_t * cr, const NodokaColors * colors,
								const WidgetParameters * widget, int x, int y,
								int width, int height)
{
	NodokaRGB *border = (NodokaRGB *) & colors->shade[3];
	NodokaRGB *fill = (NodokaRGB *) & colors->bg[widget->state_type];

	cairo_set_line_width (cr, 1.0);

	/* Fill with bg color */
	cairo_set_source_rgb (cr, fill->r, fill->g, fill->b);
	cairo_rectangle (cr, x, y, width, height);
	cairo_fill (cr);

	/* Draw border */
	cairo_rectangle (cr, x + 0.5, y + 0.5, width - 1, height - 1);
	cairo_set_source_rgba (cr, border->r, border->g, border->b, 0.8);
	cairo_stroke (cr);

	/* Draw shadow */
	cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 0.1);
	cairo_translate (cr, x + 0.5, y + 0.5);
	cairo_move_to (cr, 1, height - 2);
	cairo_line_to (cr, 1, 1);
	cairo_line_to (cr, width - 2, 1);
	cairo_stroke (cr);
}

void
nodoka_draw_progressbar_fill (cairo_t * cr, const NodokaColors * colors,
							  const WidgetParameters * widget,
							  const ProgressBarParameters * progressbar, int x,
							  int y, int width, int height, gint offset,
							  boolean stripes)
{
	boolean is_horizontal = progressbar->orientation < 2;
	double tile_pos = 0;
	double stroke_width;
	int x_step;
	NodokaRGB *fill = (NodokaRGB *) & colors->spot[1];
	NodokaRGB *border = (NodokaRGB *) & colors->spot[2];

	cairo_rectangle (cr, x, y, width, height);

	if (is_horizontal)
	{
		if (progressbar->orientation == NDK_ORIENTATION_LEFT_TO_RIGHT)
			rotate_mirror_translate (cr, 0, x, y, FALSE, FALSE);
		else
			rotate_mirror_translate (cr, 0, x + width, y, TRUE, FALSE);
	}
	else
	{
		int tmp = height;

		height = width;
		width = tmp;

		x = x + 1;
		y = y - 1;
		width = width + 2;
		height = height - 2;

		if (progressbar->orientation == NDK_ORIENTATION_TOP_TO_BOTTOM)
			rotate_mirror_translate (cr, M_PI / 2, x, y, FALSE, FALSE);
		else
			rotate_mirror_translate (cr, M_PI / 2, x, y + width, TRUE, FALSE);
	}

	cairo_save (cr);
	cairo_clip (cr);

	stroke_width = height * 2;
	x_step = (((float) stroke_width / 10) * offset);
	cairo_set_line_width (cr, 1.0);
	cairo_save (cr);

	/* Draw fill */
	cairo_rectangle (cr, 2, 1, width - 4, height - 2);
	nodoka_set_grad_full (cr, fill, DARK_TOP_HILIGHT, DARK_BOTTOM_HILIGHT,
						  GRADIENT_CENTER, 0, height, widget->gradients, FALSE,
						  1.0);
	cairo_fill (cr);

	//glass effect
	/* Draw strokes */

	if (stripes)
	{
		while (tile_pos <= width + x_step - 2)
		{
			cairo_move_to (cr, stroke_width / 2 - x_step, 0);
			cairo_line_to (cr, stroke_width - x_step, 0);
			cairo_line_to (cr, stroke_width / 2 - x_step, height);
			cairo_line_to (cr, -x_step, height);

			cairo_translate (cr, stroke_width, 0);
			tile_pos += stroke_width;
		}

		cairo_set_source_rgba (cr, border->r, border->g, border->b, 0.15);
		cairo_fill (cr);
		cairo_restore (cr);
	}

	//border
	cairo_set_source_rgba (cr, border->r, border->g, border->b, 0.8);
	cairo_rectangle (cr, 1.5, 0.5, width - 3, height - 1);
	cairo_stroke (cr);
}

void
nodoka_draw_optionmenu (cairo_t * cr,
						const NodokaColors * colors,
						const WidgetParameters * widget,
						const OptionMenuParameters * optionmenu,
						int x, int y, int width, int height)
{
	int offset = widget->ythickness + 1;

	boolean horizontal = TRUE;
	if (((float) width / height < 0.5))
		horizontal = FALSE;

	nodoka_draw_button (cr, colors, widget, x, y, width, height, horizontal,
						TRUE);

	//separator
	NodokaRGB *dark = (NodokaRGB *) & colors->shade[6];

	cairo_set_line_width (cr, 1.0);
	cairo_translate (cr, optionmenu->linepos + 0.5, 1);

	cairo_move_to (cr, 0.0, offset);
	cairo_line_to (cr, 0.0, height - offset - widget->ythickness + 1);
	cairo_set_source_rgba (cr, dark->r, dark->g, dark->b, 0.4);
	cairo_stroke (cr);
}

void
nodoka_draw_menubar (cairo_t * cr, const NodokaColors * colors,
					 const WidgetParameters * widget, int x, int y, int width,
					 int height, int menubarstyle)
{
	cairo_translate (cr, x, y);
	cairo_rectangle (cr, 0, 0, width, height);
	NodokaRGB *fill = (NodokaRGB *) & colors->bg[0];

	//glass menubar
	if (menubarstyle == 1)
	{
		double alpha = 1;
		NodokaRGB hilight;
		nodoka_shade (fill, &hilight, 1.1 * widget->hilight_ratio);
		//glass effect
		nodoka_set_grad_full (cr, fill, (widget->active ? HOLLOW_TOP_HILIGHT
										 : BULGING_TOP_HILIGHT),
							  (widget->active ? HOLLOW_BOTTOM_HILIGHT
							   : BULGING_BOTTOM_HILIGHT),
							  GRADIENT_CENTER, 0, height, widget->gradients,
							  FALSE, alpha);

		cairo_fill (cr);
	}
	//gradient menubar
	else if (menubarstyle == 2)
	{
		cairo_pattern_t *pattern;
		NodokaRGB lower;
		nodoka_shade (fill, &lower, 0.93);
		pattern = cairo_pattern_create_linear (0, 0, 0, height);
		cairo_pattern_add_color_stop_rgb (pattern, 0.0, fill->r, fill->g,
										  fill->b);
		cairo_pattern_add_color_stop_rgb (pattern, 1.0, lower.r, lower.g,
										  lower.b);
		cairo_set_source (cr, pattern);
		cairo_fill (cr);
		cairo_pattern_destroy (pattern);
	}
	//flat menubar
	else
	{
		cairo_set_source_rgb (cr, fill->r, fill->g, fill->b);
		cairo_fill (cr);
	}

}

static void
nodoka_get_frame_gap_clip (int x, int y, int width, int height,
						   FrameParameters * frame,
						   NodokaRectangle * bevel, NodokaRectangle * border)
{
	if (frame->gap_side == NDK_GAP_TOP)
	{
		NODOKA_RECTANGLE_SET ((*bevel), 1.5 + frame->gap_x, -0.5,
							  frame->gap_width - 3, 2.0);
		NODOKA_RECTANGLE_SET ((*border), 0.5 + frame->gap_x, -0.5,
							  frame->gap_width - 2, 2.0);
	}
	else if (frame->gap_side == NDK_GAP_BOTTOM)
	{
		NODOKA_RECTANGLE_SET ((*bevel), 1.5 + frame->gap_x, height - 2.5,
							  frame->gap_width - 3, 2.0);
		NODOKA_RECTANGLE_SET ((*border), 0.5 + frame->gap_x, height - 1.5,
							  frame->gap_width - 2, 2.0);
	}
	else if (frame->gap_side == NDK_GAP_LEFT)
	{
		NODOKA_RECTANGLE_SET ((*bevel), -0.5, 1.5 + frame->gap_x,
							  2.0, frame->gap_width - 3);
		NODOKA_RECTANGLE_SET ((*border), -0.5, 0.5 + frame->gap_x,
							  1.0, frame->gap_width - 2);
	}
	else if (frame->gap_side == NDK_GAP_RIGHT)
	{
		NODOKA_RECTANGLE_SET ((*bevel), width - 2.5, 1.5 + frame->gap_x,
							  2.0, frame->gap_width - 3);
		NODOKA_RECTANGLE_SET ((*border), width - 1.5, 0.5 + frame->gap_x,
							  1.0, frame->gap_width - 2);
	}
}

void
nodoka_draw_frame (cairo_t * cr,
				   const NodokaColors * colors,
				   const WidgetParameters * widget,
				   const FrameParameters * frame,
				   int x, int y, int width, int height)
{
	NodokaRGB *border = frame->border;
	NodokaRectangle bevel_clip = { 0, 0, 0, 0 };	// fix compiler warning
	NodokaRectangle frame_clip = { 0, 0, 0, 0 };	// fix compiler warning

	NodokaRGB *dark = (NodokaRGB *) & colors->shade[3];
	NodokaRGB hilight;
	nodoka_shade (dark, &hilight, 1.3);

	if (frame->shadow == NDK_SHADOW_NONE)
		return;

	if (frame->gap_x != -1)
		nodoka_get_frame_gap_clip (x, y, width, height,
								   (FrameParameters *) frame,
								   &bevel_clip, &frame_clip);

	cairo_set_line_width (cr, 1.0);
	cairo_translate (cr, x + 0.5, y + 0.5);

	/* save everything */
	cairo_save (cr);

	/* Set clip for the bevel */
	if (frame->gap_x != -1)
	{
		/* Set clip for gap */
		cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
		cairo_rectangle (cr, -0.5, -0.5, width, height);
		cairo_rectangle (cr, bevel_clip.x, bevel_clip.y, bevel_clip.width,
						 bevel_clip.height);
		cairo_clip (cr);
	}

	/* Draw the bevel */
	if (frame->shadow == NDK_SHADOW_ETCHED_IN
		|| frame->shadow == NDK_SHADOW_ETCHED_OUT)
	{
		cairo_set_source_rgb (cr, hilight.r, hilight.g, hilight.b);
		if (frame->shadow == NDK_SHADOW_ETCHED_IN)
			nodoka_rounded_rectangle (cr, 1, 1, width - 2, height - 2,
									  widget->roundness, widget->corners);
		else
			nodoka_rounded_rectangle (cr, 0, 0, width - 2, height - 2,
									  widget->roundness, widget->corners);
		cairo_stroke (cr);
	}
	else if (frame->shadow != NDK_SHADOW_NONE
			 && frame->shadow != NDK_SHADOW_FLAT)
	{
		ShadowParameters shadow;
		shadow.corners = widget->corners;
		shadow.shadow = frame->shadow;
		if (!(frame->shadow & NDK_SHADOW_OUT))
		{
			cairo_move_to (cr, 1, height - 2);
			cairo_line_to (cr, 1, 1);
			cairo_line_to (cr, width - 1.5, 1);
			cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 0.15);
			cairo_stroke (cr);
		}
		else
		{
			cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 0.15);
			nodoka_rounded_rectangle (cr, 0, 0, width - 1, height - 1,
									  widget->roundness + 1, widget->corners);
			cairo_stroke (cr);
		}


	}

	/* restore the previous clip region */
	cairo_restore (cr);
	cairo_save (cr);
	if (frame->gap_x != -1)
	{
		/* Set clip for gap */
		cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
		cairo_rectangle (cr, -0.5, -0.5, width, height);
		cairo_rectangle (cr, frame_clip.x, frame_clip.y, frame_clip.width,
						 frame_clip.height);
		cairo_clip (cr);
	}

	/* Draw frame */
	if (frame->shadow == NDK_SHADOW_ETCHED_IN
		|| frame->shadow == NDK_SHADOW_ETCHED_OUT)
	{
		cairo_set_source_rgb (cr, dark->r, dark->g, dark->b);
		if (frame->shadow == NDK_SHADOW_ETCHED_IN)
			nodoka_rounded_rectangle (cr, 0, 0, width - 2, height - 2,
									  widget->roundness, widget->corners);
		else
			nodoka_rounded_rectangle (cr, 1, 1, width - 2, height - 2,
									  widget->roundness, widget->corners);
	}
	else
	{
		cairo_set_source_rgb (cr, border->r, border->g, border->b);
		if (frame->shadow & NDK_SHADOW_OUT)
			nodoka_rounded_rectangle (cr, 0, 0, width - 2, height - 2,
									  widget->roundness, widget->corners);
		else
			nodoka_rounded_rectangle (cr, 0, 0, width - 1, height - 1,
									  widget->roundness, widget->corners);
	}
	cairo_stroke (cr);
	cairo_restore (cr);
}

void
nodoka_draw_tab (cairo_t * cr,
				 const NodokaColors * colors,
				 const WidgetParameters * widget,
				 const TabParameters * tab, int x, int y, int width, int height)
{
	const float RADIUS = 3.0;
	int corners;
	double strip_size;
	NodokaRGB *stripe_fill = (NodokaRGB *) & colors->spot[1];
	NodokaRGB *stripe_border = (NodokaRGB *) & colors->spot[2];
	NodokaRGB *fill;
	NodokaRGB *border1;

	if (widget->active)
		fill = (NodokaRGB *) & colors->bg[widget->state_type];
	else
		fill = (NodokaRGB *) & widget->parentbg;

	if (!widget->active)
		border1 = (NodokaRGB *) & colors->shade[5];
	else
		border1 = (NodokaRGB *) & colors->shade[4];

	cairo_pattern_t *pattern;

	/* Set clip */
	cairo_rectangle (cr, x, y, width, height);
	cairo_clip (cr);
	cairo_new_path (cr);

	/* Translate and set line width */
	cairo_set_line_width (cr, 1.0);
	cairo_translate (cr, x + 0.5, y + 0.5);

	/* Make the tabs slightly bigger than they should be, to create a gap */
	/* And calculate the strip size too, while you're at it */
	if (tab->gap_side == NDK_GAP_TOP || tab->gap_side == NDK_GAP_BOTTOM)
	{
		height += RADIUS;
		strip_size = ((100.0 / height) * 2.0) / 100.0;

		if (tab->gap_side == NDK_GAP_TOP)
		{
			cairo_translate (cr, 0.0, -4.0);	/* gap at the other side */
			corners = NDK_CORNER_BOTTOMLEFT | NDK_CORNER_BOTTOMRIGHT;
		}
		else
			corners = NDK_CORNER_TOPLEFT | NDK_CORNER_TOPRIGHT;
	}
	else
	{
		width += RADIUS;
		strip_size = ((100.0 / width) * 2.0) / 100.0;

		if (tab->gap_side == NDK_GAP_LEFT)
		{
			cairo_translate (cr, -4.0, 0.0);	/* gap at the other side */
			corners = NDK_CORNER_TOPRIGHT | NDK_CORNER_BOTTOMRIGHT;
		}
		else
			corners = NDK_CORNER_TOPLEFT | NDK_CORNER_BOTTOMLEFT;
	}

	/* Set tab shape */
	if (widget->roundness < 2)
		cairo_rectangle (cr, 0, 0, width - 1, height - 1);
	else
		nodoka_rounded_rectangle (cr, 0, 0, width - 1, height - 1,
								  widget->roundness, corners);


	if (widget->active)
	{
		pattern =
			cairo_pattern_create_linear (tab->gap_side ==
										 NDK_GAP_LEFT ? width - 1 : 0,
										 tab->gap_side ==
										 NDK_GAP_TOP ? height - 2 : 1,
										 tab->gap_side ==
										 NDK_GAP_RIGHT ? width : 0,
										 tab->gap_side ==
										 NDK_GAP_BOTTOM ? height : 0);

		NodokaRGB shadow;
		nodoka_shade (fill, &shadow, HOLLOW_TOP_HILIGHT);

		cairo_pattern_add_color_stop_rgb (pattern, 0.0, shadow.r, shadow.g,
										  shadow.b);
		cairo_pattern_add_color_stop_rgb (pattern, GRADIENT_CENTER, fill->r,
										  fill->g, fill->b);
		cairo_pattern_add_color_stop_rgb (pattern, 1.0, fill->r, fill->g,
										  fill->b);
		cairo_set_source (cr, pattern);
		cairo_fill (cr);
		cairo_pattern_destroy (pattern);

		cairo_stroke (cr);
	}
	else
	{
		/* Draw shade */
		pattern =
			cairo_pattern_create_linear (tab->gap_side ==
										 NDK_GAP_LEFT ? width - 2 : 0,
										 tab->gap_side ==
										 NDK_GAP_TOP ? height - 2 : 0,
										 tab->gap_side ==
										 NDK_GAP_RIGHT ? width : 0,
										 tab->gap_side ==
										 NDK_GAP_BOTTOM ? height : 0);


		cairo_pattern_add_color_stop_rgb (pattern, 0.0, stripe_fill->r,
										  stripe_fill->g, stripe_fill->b);
		cairo_pattern_add_color_stop_rgb (pattern, strip_size, stripe_fill->r,
										  stripe_fill->g, stripe_fill->b);
		cairo_pattern_add_color_stop_rgb (pattern, strip_size, colors->bg[0].r,
										  colors->bg[0].g, colors->bg[0].b);
		cairo_set_source (cr, pattern);
		cairo_fill (cr);
		cairo_pattern_destroy (pattern);

		int x1, x2, y1, y2;
		if (tab->gap_side == NDK_GAP_LEFT || tab->gap_side == NDK_GAP_RIGHT)
		{
			y1 = 0;
			y2 = 0;
			x2 = tab->gap_side == NDK_GAP_LEFT ? width - 15 : 15;	//width/2
			x1 = tab->gap_side == NDK_GAP_LEFT ? width : 0;
		}
		else
		{
			x1 = 0;
			x2 = 0;
			y2 = tab->gap_side == NDK_GAP_TOP ? height - 15 : 15;	//height/2
			y1 = tab->gap_side == NDK_GAP_TOP ? height : 0;
		}
		pattern = cairo_pattern_create_linear (x1, y1, x2, y2);

		cairo_pattern_add_color_stop_rgba (pattern, 0.0, stripe_fill->r,
										   stripe_fill->g, stripe_fill->b, 0.2);
		cairo_pattern_add_color_stop_rgba (pattern, 1.0, fill->r, fill->g,
										   fill->b, 0.0);
		cairo_set_source (cr, pattern);

		if (widget->roundness < 2)
			cairo_rectangle (cr, 0, 0, width - 1, height - 1);
		else
			clearlooks_rounded_rectangle (cr, 0, 0, width - 1, height - 1,
										  widget->roundness, corners);

		cairo_fill (cr);
		cairo_pattern_destroy (pattern);
	}

	if (widget->corners == NDK_CORNER_NONE)
		cairo_rectangle (cr, 0, 0, width - 1, height - 1);
	else
		nodoka_rounded_rectangle (cr, 0, 0, width - 1, height - 1,
								  widget->roundness, corners);

	if (widget->active)
	{
		cairo_set_source_rgb (cr, border1->r, border1->g, border1->b);
		cairo_stroke (cr);
	}
	else
	{
		pattern =
			cairo_pattern_create_linear (tab->gap_side ==
										 NDK_GAP_LEFT ? width - 2 : 2,
										 tab->gap_side ==
										 NDK_GAP_TOP ? height - 2 : 2,
										 tab->gap_side ==
										 NDK_GAP_RIGHT ? width : 2,
										 tab->gap_side ==
										 NDK_GAP_BOTTOM ? height : 2);

		cairo_pattern_add_color_stop_rgb (pattern, 0.0, stripe_border->r,
										  stripe_border->g, stripe_border->b);
		cairo_pattern_add_color_stop_rgb (pattern, strip_size, stripe_border->r,
										  stripe_border->g, stripe_border->b);
		cairo_pattern_add_color_stop_rgb (pattern, strip_size, stripe_border->r,
										  stripe_border->g, stripe_border->b);
		cairo_pattern_add_color_stop_rgb (pattern, 0.6, border1->r, border1->g,
										  border1->b);
		cairo_set_source (cr, pattern);
		cairo_stroke (cr);
		cairo_pattern_destroy (pattern);
	}
}

void
nodoka_draw_separator (cairo_t * cr, const NodokaColors * colors,
					   const WidgetParameters * widget,
					   const SeparatorParameters * separator, int x, int y,
					   int width, int height)
{
	NodokaRGB *dark = (NodokaRGB *) & colors->shade[6];

	if (separator->horizontal)
	{
		cairo_set_line_width (cr, 1.0);
		cairo_translate (cr, x, y + 0.5);

		cairo_move_to (cr, 0.0, 0.0);
		cairo_line_to (cr, width + 1, 0.0);
		cairo_set_source_rgba (cr, dark->r, dark->g, dark->b, 0.4);
		cairo_stroke (cr);
	}
	else
	{
		cairo_set_line_width (cr, 1.0);
		cairo_translate (cr, x + 0.5, y);

		cairo_move_to (cr, 0.0, 0.0);
		cairo_line_to (cr, 0.0, height);
		cairo_set_source_rgba (cr, dark->r, dark->g, dark->b, 0.4);
		cairo_stroke (cr);
	}
}

void
nodoka_draw_combo_separator (cairo_t * cr,
							 const NodokaColors * colors,
							 const WidgetParameters * widget,
							 int x, int y, int width, int height)
{
	NodokaRGB *dark = (NodokaRGB *) & colors->shade[6];

	cairo_set_line_width (cr, 1.0);
	cairo_translate (cr, x + 0.5, y);

	cairo_move_to (cr, 0.0, 0.0);
	cairo_line_to (cr, 0.0, height + 1);
	cairo_set_source_rgba (cr, dark->r, dark->g, dark->b, 0.4);
	cairo_stroke (cr);
}

void
nodoka_draw_list_view_header (cairo_t * cr,
							  const NodokaColors * colors,
							  const WidgetParameters * widget,
							  const ListViewHeaderParameters * header,
							  int x, int y, int width, int height)
{
	const NodokaRGB *fill = &colors->bg[widget->state_type];
	const NodokaRGB *border = &colors->shade[3];
	NodokaRGB hilight;
	nodoka_shade (border, &hilight, 1.3);

	cairo_translate (cr, x, y);
	cairo_set_line_width (cr, 1.0);

	if (header->order == NDK_ORDER_FIRST)
	{
		cairo_move_to (cr, 0.5, height - 1);
		cairo_line_to (cr, 0.5, 0.5);
	}
	else
		cairo_move_to (cr, 0.0, 0.5);

	cairo_line_to (cr, width, 0.5);
	cairo_set_source_rgb (cr, hilight.r, hilight.g, hilight.b);
	cairo_stroke (cr);

	//effects
	if (header->style > 0)
	{
		NodokaRGB hilight_header;
		nodoka_shade (fill, &hilight_header,
					  GLASS_HILIGHT * widget->hilight_ratio);
		//glassy header
		if (header->style == 1)
		{
			cairo_rectangle (cr, 0, 0, width, height);
			nodoka_set_grad_full (cr, fill, BULGING_TOP_HILIGHT,
								  BULGING_BOTTOM_HILIGHT, GRADIENT_CENTER,
								  0, height, widget->gradients, FALSE, 1);
			//glass effect
			cairo_fill (cr);

			nodoka_set_gradient (cr, &hilight_header, GLASS_HILIGHT, 0, height,
								 widget->gradients, TRUE);
			cairo_fill (cr);
		}
		//raised
		else if (header->style == 2)
		{
			border = (NodokaRGB *) & colors->shade[4];
			NodokaRGB shadow_header;
			nodoka_shade (fill, &shadow_header, 0.925);

			if (!widget->gradients)
			{
				cairo_set_source_rgb (cr, shadow_header.r, shadow_header.g,
									  shadow_header.b);
				cairo_rectangle (cr, 0.0, height - 3.0, width, 2.0);
			}
			else
			{
				cairo_pattern_t *pattern;
				pattern = cairo_pattern_create_linear (0.0, height - 4.0, 0.0,
													   height - 1.0);
				cairo_pattern_add_color_stop_rgba (pattern, 0.0,
												   shadow_header.r,
												   shadow_header.g,
												   shadow_header.b, 0.0);
				cairo_pattern_add_color_stop_rgba (pattern, 1.0,
												   shadow_header.r,
												   shadow_header.g,
												   shadow_header.b, 1);
				cairo_set_source (cr, pattern);
				cairo_pattern_destroy (pattern);
				cairo_rectangle (cr, 0.0, height - 4.0, width, 3.0);
			}
			cairo_fill (cr);
		}
	}
	/* Draw bottom border */
	cairo_move_to (cr, 0.0, height - 0.5);
	cairo_line_to (cr, width, height - 0.5);
	cairo_set_source_rgb (cr, border->r, border->g, border->b);
	cairo_stroke (cr);

	/* Draw resize grip */
	if (header->order != NDK_ORDER_LAST || header->resizable)
	{
		SeparatorParameters separator;
		separator.horizontal = FALSE;

		nodoka_draw_separator (cr, colors, widget, &separator, width - 0.5, 4.0,
							   1, height - 8.0);
	}
}

/* We can't draw transparent things here, since it will be called on the same
 * surface multiple times, when placed on a handlebox_bin or dockitem_bin */
void
nodoka_draw_toolbar (cairo_t * cr, const NodokaColors * colors,
					 const WidgetParameters * widget, int x, int y, int width,
					 int height)
{
	NodokaRGB *dark = (NodokaRGB *) & colors->shade[3];
	NodokaRGB hilight;

	nodoka_shade (dark, &hilight, 1.3);

	cairo_set_line_width (cr, 1.0);
	cairo_translate (cr, x, y);

	/* Draw shadow */
	cairo_move_to (cr, 0, height - 0.5);
	cairo_line_to (cr, width - 1, height - 0.5);
	cairo_set_source_rgb (cr, dark->r, dark->g, dark->b);
	cairo_stroke (cr);
}

void
nodoka_draw_menuitem (cairo_t * cr,
					  const NodokaColors * colors,
					  const WidgetParameters * widget,
					  int x, int y, int width, int height, boolean stripes)
{
	NodokaRGB *fill = (NodokaRGB *) & colors->spot[1];
	NodokaRGB *border = (NodokaRGB *) & colors->spot[2];
	NodokaRGB hilight;
	nodoka_shade (fill, &hilight, 1.1 * widget->hilight_ratio);

	cairo_translate (cr, x, y);
	cairo_set_line_width (cr, 1.0);
	if (widget->roundness < 2)
		cairo_rectangle (cr, 0, 0, width, height);
	else
		clearlooks_rounded_rectangle (cr, 0, 0, width, height,
									  widget->roundness, widget->corners);

	nodoka_set_grad_full (cr, fill, DARK_TOP_HILIGHT, DARK_BOTTOM_HILIGHT,
						  GRADIENT_CENTER, 0, height, widget->gradients,
						  FALSE, 1.0);

	//do we use stripes?
	if (stripes)
	{
		if (widget->roundness > 1)
			cairo_clip_preserve (cr);
		cairo_fill (cr);
		double tile_pos = 0;
		double stroke_width;
		int x_step;
		stroke_width = height * 2;
		cairo_save (cr);
		x_step = (((float) stroke_width / 10));
		/* Draw strokes */
		while (tile_pos <= width + x_step - 2)
		{
			cairo_move_to (cr, stroke_width / 2 - x_step, 0);
			cairo_line_to (cr, stroke_width - x_step, 0);
			cairo_line_to (cr, stroke_width / 2 - x_step, height);
			cairo_line_to (cr, -x_step, height);
			cairo_translate (cr, stroke_width, 0);
			tile_pos += stroke_width;
		}
		cairo_set_source_rgba (cr, border->r, border->g, border->b, 0.15);
		cairo_fill (cr);
		cairo_restore (cr);
	}
	else
	{
		if (widget->roundness > 1)
			cairo_clip_preserve (cr);
		//glass effect
		cairo_fill (cr);

		nodoka_set_gradient (cr, &hilight, GLASS_HILIGHT, 0, height,
							 widget->gradients, TRUE);
		cairo_fill (cr);
	}

	nodoka_rounded_rectangle (cr, 0.5, 0.5, width - 1, height - 1,
							  widget->roundness, widget->corners);
	cairo_set_source_rgba (cr, border->r, border->g, border->b, 0.8);
	cairo_stroke (cr);
}

void
nodoka_draw_scrollbar_trough (cairo_t * cr, const NodokaColors * colors,
							  const WidgetParameters * widget,
							  const ScrollBarParameters * scrollbar, int x,
							  int y, int width, int height)
{
	NodokaRGB *bg = (NodokaRGB *) & colors->bg[widget->state_type];
	NodokaRGB *border = (NodokaRGB *) & colors->shade[5];

	cairo_set_line_width (cr, 1);

	if (scrollbar->horizontal)
	{
		int tmp = height;

		rotate_mirror_translate (cr, M_PI / 2, x, y, FALSE, FALSE);
		height = width;
		width = tmp;
	}
	else
	{
		cairo_translate (cr, x, y);
	}

	/* Draw fill */
	if (widget->roundness < 2)
		cairo_rectangle (cr, 1, 0, width - 2, height);
	else
		clearlooks_rounded_rectangle (cr, 1, 0, width - 2, height,
									  widget->roundness, widget->corners);
	cairo_set_source_rgb (cr, bg->r, bg->g, bg->b);
	cairo_fill (cr);

	/* Draw border */
	if (widget->roundness < 2)
		cairo_rectangle (cr, 0.5, 0.5, width - 1, height - 1);
	else
		clearlooks_rounded_rectangle (cr, 0.5, 0.5, width - 1, height - 1,
									  widget->roundness, widget->corners);
	cairo_set_source_rgba (cr, border->r, border->g, border->b, 0.8);
	cairo_stroke (cr);

	/* Draw shadow */
	cairo_move_to (cr, 1.5, height - 1.5);
	cairo_line_to (cr, 1.5, 1.5);
	cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 0.1);
	cairo_stroke (cr);
}

void
nodoka_draw_scrollbar_stepper (cairo_t * cr, const NodokaColors * colors,
							   const WidgetParameters * widget,
							   const ScrollBarParameters * scrollbar,
							   const ScrollBarStepperParameters * stepper,
							   int x, int y, int width, int height)
{
	const NodokaRGB *fill = &colors->bg[widget->state_type];
	NodokaRGB border_normal;

	nodoka_shade (&colors->shade[6], &border_normal, 0.95);
	//the border
	border_normal.r = border_normal.r * (0.6) + fill->r * 0.4;
	border_normal.g = border_normal.g * (0.6) + fill->g * 0.4;
	border_normal.b = border_normal.b * (0.6) + fill->b * 0.4;

	cairo_translate (cr, x, y);
	cairo_set_line_width (cr, 1.0);

	//bg
	if (widget->roundness < 2)
		cairo_rectangle (cr, 1, 1, width - 2, height - 2);
	else
		clearlooks_rounded_rectangle (cr, 0.5, 0.5, width - 1, height - 1,
									  widget->roundness, widget->corners);

	nodoka_set_grad_full (cr, fill, BULGING_TOP_HILIGHT, BULGING_BOTTOM_HILIGHT,
						  GRADIENT_CENTER, 0, height, widget->gradients, FALSE,
						  1.0);
	cairo_fill (cr);

	//border
	cairo_set_source_rgb (cr, border_normal.r, border_normal.g,
						  border_normal.b);
	nodoka_rounded_rectangle (cr, 0.5, 0.5, width - 1, height - 1,
							  widget->roundness, widget->corners);
	cairo_stroke (cr);

	//shadow
	if (!(widget->disabled))
	{
		if (((stepper->stepper == NDK_STEPPER_A) && (!(scrollbar->steppers
													   & NDK_STEPPER_B)))
			|| (stepper->stepper == NDK_STEPPER_B))
		{
			cairo_move_to (cr, (scrollbar->horizontal ? width + 0.5 : 1.5),
						   (scrollbar->horizontal ? 1.5 : height + 0.5));
			cairo_line_to (cr,
						   (scrollbar->horizontal ? width + 0.5 : width - 0.5),
						   (scrollbar->horizontal ? height - 0.5 : height +
							0.5));
			cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 0.1);
			cairo_stroke (cr);
		}
	}
}

void
nodoka_draw_scrollbar_slider (cairo_t * cr, const NodokaColors * colors,
							  const WidgetParameters * widget,
							  const ScrollBarParameters * scrollbar, int x,
							  int y, int width, int height)
{
	if (scrollbar->junction & NDK_JUNCTION_BEGIN)
	{
		if (scrollbar->horizontal)
		{
			x -= 1;
			width += 1;
		}
		else
		{
			y -= 1;
			height += 1;
		}
	}
	if (scrollbar->junction & NDK_JUNCTION_END)
	{
		if (scrollbar->horizontal)
			width += 1;
		else
			height += 1;
	}
	//setting colors

	const NodokaRGB *fill = &colors->bg[widget->state_type];
	NodokaRGB border;

	nodoka_shade (&colors->shade[6], &border, 0.95);

	//the border
	border.r = border.r * (0.6) + fill->r * 0.4;
	border.g = border.g * (0.6) + fill->g * 0.4;
	border.b = border.b * (0.6) + fill->b * 0.4;

	if (scrollbar->horizontal)
		cairo_translate (cr, x, y);
	else
	{
		int tmp = height;

		rotate_mirror_translate (cr, M_PI / 2, x, y, FALSE, FALSE);
		height = width;
		width = tmp;
	}

	cairo_set_line_width (cr, 1);

	/* Draw background */
	cairo_rectangle (cr, 1, 1, width - 2, height - 2);
	nodoka_set_grad_full (cr, fill, BULGING_TOP_HILIGHT, BULGING_BOTTOM_HILIGHT,
						  GRADIENT_CENTER, 0, height, widget->gradients, FALSE,
						  1.0);
	cairo_fill (cr);

	/* Draw border */
	cairo_rectangle (cr, 0.5, 0.5, width - 1, height - 1);
	cairo_set_source_rgb (cr, border.r, border.g, border.b);
	cairo_stroke (cr);

	/* Draw shadow */
	if (!(scrollbar->junction & NDK_JUNCTION_END))
	{
		cairo_move_to (cr, width + 0.5, 1.5);
		cairo_line_to (cr, width + 0.5, height - 1);
		cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 0.1);
		cairo_stroke (cr);
	}

	//Options
	NodokaRGB style = colors->shade[5];

	//Circles
	if (scrollbar->style % 3 == 1)
	{
		int circ_radius = 1;
		int circ_space = 4;
		int i;
		float circ_y = height / 2.0 - (circ_space / 2.0);

		float circ_x = width / 2.0 - (circ_space);

		cairo_translate (cr, 0.5, 0.5);

		for (i = 0; i < 3; i++)
		{
			cairo_move_to (cr, circ_x, circ_y);

			cairo_arc (cr, circ_x, circ_y, circ_radius, 0, M_PI * 2);
			cairo_close_path (cr);

			cairo_arc (cr, circ_x, circ_y + circ_space, circ_radius, 0,
					   M_PI * 2);
			cairo_close_path (cr);

			cairo_set_source_rgba (cr, style.r, style.g, style.b, 0.5);
			cairo_fill (cr);

			circ_x += circ_space;
		}
	}
	//Handle        
	if (scrollbar->style % 3 == 2)
	{
		int bar_x = width / 2 - 4;

		cairo_translate (cr, 0.5, 0.5);
		int i;

		for (i = 0; i < 3; i++)
		{
			cairo_move_to (cr, bar_x, 4.5);
			cairo_line_to (cr, bar_x, height - 5.5);
			cairo_set_source_rgba (cr, border.r, border.g, border.b, 0.5);
			cairo_stroke (cr);

			bar_x += 3;
		}
	}
}

void
nodoka_draw_selected_cell (cairo_t * cr,
						   const NodokaColors * colors,
						   const WidgetParameters * widget,
						   int x, int y, int width, int height, boolean stripes)
{
	cairo_set_line_cap (cr, CAIRO_LINE_CAP_SQUARE);
	cairo_set_line_join (cr, CAIRO_LINE_JOIN_MITER);

	NodokaRGB fill;
	cairo_save (cr);

	cairo_translate (cr, x, y);

	if (widget->focus)
		fill = colors->base[widget->state_type];
	else
		fill = colors->base[GTK_STATE_ACTIVE];

	nodoka_set_grad_full (cr, &fill, BULGING_TOP_HILIGHT,
						  BULGING_BOTTOM_HILIGHT, GRADIENT_CENTER, 0,
						  height, widget->gradients, FALSE, 1.0);
	cairo_rectangle (cr, 0, 0, width, height);
	cairo_fill (cr);

	NodokaRGB border;
	nodoka_shade (&fill, &border, (!widget->gradients ? 0.9 : 0.95));

	cairo_move_to (cr, 0, 0.5);
	cairo_rel_line_to (cr, width, 0);
	cairo_move_to (cr, 0, height - 0.5);
	cairo_rel_line_to (cr, width, 0);

	cairo_set_source_rgb (cr, border.r, border.g, border.b);
	cairo_stroke (cr);
	cairo_restore (cr);
}

void
nodoka_draw_statusbar (cairo_t * cr,
					   const NodokaColors * colors,
					   const WidgetParameters * widget,
					   int x, int y, int width, int height)
{
	NodokaRGB *dark = (NodokaRGB *) & colors->shade[3];

	cairo_set_line_width (cr, 1);
	cairo_translate (cr, x, y + 0.5);
	cairo_move_to (cr, 0, 0);
	cairo_line_to (cr, width, 0);
	cairo_set_source_rgb (cr, dark->r, dark->g, dark->b);
	cairo_stroke (cr);
}

void
nodoka_draw_menu_frame (cairo_t * cr,
						const NodokaColors * colors,
						const WidgetParameters * widget,
						int x, int y, int width, int height)
{
	NodokaRGB *border = (NodokaRGB *) & colors->shade[5];
	cairo_translate (cr, x, y);
	cairo_set_line_width (cr, 1);
	cairo_rectangle (cr, 0.5, 0.5, width - 1, height - 1);
	cairo_set_source_rgb (cr, border->r, border->g, border->b);

	cairo_stroke (cr);

}

void
nodoka_draw_handle (cairo_t * cr, const NodokaColors * colors,
					const WidgetParameters * widget,
					const HandleParameters * handle, int x, int y, int width,
					int height)
{
	NodokaRGB *dark = (NodokaRGB *) & colors->shade[3];

	int bar_height;
	int i;
	int num_bars, bar_spacing;

	num_bars = 3;
	bar_spacing = 3;
	bar_height = num_bars * bar_spacing;

	if (handle->horizontal)
	{
		int tmp = height;

		rotate_mirror_translate (cr, M_PI / 2, x + 0.5, y + 0.5, FALSE, FALSE);
		height = width;
		width = tmp;
	}
	else
	{
		cairo_translate (cr, x, y);
	}

	int circ_radius = 1;
	int circ_space = 6;

	float circ_y = height / 2.0 - circ_space;
	float circ_x = width / 2.0;	//width/2 - (circ_radius/2);

	for (i = 0; i < 3; i++)
	{
		cairo_move_to (cr, circ_x, circ_y);

		cairo_arc (cr, circ_x, circ_y, circ_radius + 0.5, 0, M_PI * 2);
		cairo_close_path (cr);

		cairo_set_source_rgba (cr, dark->r, dark->g, dark->b, 0.4);
		cairo_fill (cr);

		cairo_arc (cr, circ_x, circ_y, circ_radius, 0, M_PI * 2);
		cairo_close_path (cr);

		cairo_set_source_rgba (cr, dark->r, dark->g, dark->b, 0.4);
		cairo_fill (cr);
		circ_y += circ_space;
	}
}

static void
nodoka_draw_normal_arrow (cairo_t * cr, NodokaRGB * color,
						  double x, double y, double width, double height)
{
	const float ARROW_WIDTH = 7.7;
	const int ARROW_HEIGHT = 4.0;

	cairo_set_line_width (cr, 1);

	cairo_move_to (cr, -ARROW_WIDTH / 2, -ARROW_HEIGHT / 2);
	cairo_line_to (cr, 0, ARROW_HEIGHT / 2);
	cairo_line_to (cr, ARROW_WIDTH / 2, -ARROW_HEIGHT / 2);

	cairo_set_source_rgb (cr, color->r, color->g, color->b);
	cairo_stroke (cr);
}

static void
nodoka_draw_small_arrow (cairo_t * cr, NodokaRGB * color, double x, double y,
						 double width, double height)
{
	const float ARROW_WIDTH = 5.5;
	const int ARROW_HEIGHT = 3;

	cairo_set_line_width (cr, 0.8);

	cairo_move_to (cr, -ARROW_WIDTH / 2.0, -ARROW_HEIGHT / 2.0);
	cairo_line_to (cr, 0, ARROW_HEIGHT / 2.0);
	cairo_line_to (cr, ARROW_WIDTH / 2.0, -ARROW_HEIGHT / 2.0);

	cairo_set_source_rgb (cr, color->r, color->g, color->b);
	cairo_stroke (cr);
}

static void
nodoka_draw_combo_arrow (cairo_t * cr, NodokaRGB * color,
						 double x, double y, double width, double height)
{
	const float ARROW_WIDTH = 7.7;
	const int ARROW_HEIGHT = 4.0;
	const int ARROW_SPACING = 8;

	cairo_set_line_width (cr, 1);

	y -= ARROW_SPACING / 2;

	cairo_move_to (cr, -ARROW_WIDTH / 2, y + ARROW_HEIGHT / 2);
	cairo_line_to (cr, 0, y + -ARROW_HEIGHT / 2);
	cairo_line_to (cr, ARROW_WIDTH / 2, y + ARROW_HEIGHT / 2);
	cairo_set_source_rgb (cr, color->r, color->g, color->b);
	cairo_stroke (cr);

	y += ARROW_SPACING;

	cairo_move_to (cr, -ARROW_WIDTH / 2, y + -ARROW_HEIGHT / 2);
	cairo_line_to (cr, 0, y + ARROW_HEIGHT / 2);
	cairo_line_to (cr, ARROW_WIDTH / 2, y + -ARROW_HEIGHT / 2);
	cairo_set_source_rgb (cr, color->r, color->g, color->b);
	cairo_stroke (cr);
}

static void
_nodoka_draw_arrow (cairo_t * cr, NodokaRGB * color,
					NodokaDirection dir, NodokaArrowType type,
					double x, double y, double width, double height)
{
	double rotate;

	if (dir == NDK_DIRECTION_LEFT)
		rotate = M_PI * 1.5;
	else if (dir == NDK_DIRECTION_RIGHT)
		rotate = M_PI * 0.5;
	else if (dir == NDK_DIRECTION_UP)
		rotate = M_PI;
	else
		rotate = 0;

	if (type == NDK_ARROW_NORMAL || type == NDK_ARROW_SCROLL)
	{
		rotate_mirror_translate (cr, rotate, x, y, FALSE, FALSE);
		nodoka_draw_normal_arrow (cr, color, 0, 0, width, height);
	}
	else if (type == NDK_ARROW_COMBO)
	{
		cairo_translate (cr, x, y);
		nodoka_draw_combo_arrow (cr, color, 0, 0, width, height);
	}
	else if (type == NDK_ARROW_SPINBUTTON)
	{
		rotate_mirror_translate (cr, rotate, x, y, FALSE, FALSE);
		nodoka_draw_small_arrow (cr, color, 0, 0, width, height);
	}
}

void
nodoka_draw_arrow (cairo_t * cr, const NodokaColors * colors,
				   const WidgetParameters * widget,
				   const ArrowParameters * arrow, int x, int y, int width,
				   int height)
{
	NodokaRGB *color;

	color = (NodokaRGB *) & colors->text[widget->state_type];
	gdouble tx, ty;

	if (arrow->direction == NDK_DIRECTION_DOWN
		|| arrow->direction == NDK_DIRECTION_UP)
	{
		tx = x + width / 2;
		ty = (y + height / 2) + 0.5;
	}
	else
	{
		tx = (x + width / 2) + 0.5;
		ty = y + height / 2;
	}

	if (widget->disabled)
	{
		_nodoka_draw_arrow (cr, (NodokaRGB *) & colors->shade[0],
							arrow->direction, arrow->type, tx + 0.5, ty + 0.5,
							width, height);

		color = (NodokaRGB *) & colors->text[widget->state_type];
	}

	cairo_identity_matrix (cr);

	_nodoka_draw_arrow (cr, color, arrow->direction, arrow->type, tx, ty, width,
						height);
}

void
nodoka_draw_radiobutton (cairo_t * cr, const NodokaColors * colors,
						 const WidgetParameters * widget,
						 const OptionParameters * status, int x, int y,
						 int width, int height, double trans)
{
	const NodokaRGB *border;
	NodokaRGB dot;

    int sx = x + width/2;
    int sy = y + height/2;
    
	width = height = 15;
	
	sx = sx - width/2;
	sy = sy - height/2;

	if (widget->state_type == GTK_STATE_INSENSITIVE)
	{
		border = &colors->shade[3];
		dot = colors->shade[3];
	}
	else
	{
		border = &colors->shade[5];
		if (widget->prelight)
			border = &colors->spot[1];
		dot.r = 0;
		dot.g = 93 / 256.0;
		dot.b = 194 / 256.0;
	}

	cairo_translate (cr, sx, sy);

	/* Draw backround */
	cairo_set_line_width (cr, 1);
	if (widget->state_type != GTK_STATE_INSENSITIVE)
	{
		const NodokaRGB *bg = &colors->base[0];
		cairo_arc (cr, 7, 7, 5, 0, M_PI * 2);
		cairo_set_source_rgb (cr, bg->r, bg->g, bg->b);
		cairo_fill (cr);
	}

	/* Draw border */
	cairo_arc (cr, 7, 7, 5.5, 0, M_PI * 2);
	cairo_set_source_rgba (cr, border->r, border->g, border->b, 0.8);
	cairo_stroke (cr);

	/* Draw bullet */
	if (status->draw_bullet && (!(status->inconsistent)))
	{
		cairo_arc (cr, 7, 7, 3, 0, M_PI * 2);
		cairo_set_source_rgba (cr, dot.r, dot.g, dot.b, trans);
		cairo_fill (cr);
	}

	/* Draw inconsistent */
	if (status->inconsistent)
	{
		cairo_set_line_width (cr, 2);
		cairo_move_to (cr, 3, 7);
		cairo_line_to (cr, 11, 7);
		cairo_set_source_rgba (cr, dot.r, dot.g, dot.b, trans);
		cairo_stroke (cr);
	}

	/* Draw shadow */
	if (!(widget->disabled))
	{
		cairo_arc (cr, 7, 7, 5, M_PI * 2 / 3.0, M_PI * 16 / 9.0);
		cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 0.2);
		cairo_stroke (cr);
	}
}

void
nodoka_draw_checkbutton (cairo_t * cr,
						 const NodokaColors * colors,
						 const WidgetParameters * widget,
						 const OptionParameters * status,
						 int x, int y, int width, int height, double trans)
{
	const NodokaRGB *border;
	NodokaRGB dot;
	
    int sx = x + width/2;
    int sy = y + height/2;
    
	width = height = 13;
	
	sx = sx - height/2;
	sy = sy - height/2;

	if (widget->state_type == GTK_STATE_INSENSITIVE)
	{
		border = &colors->shade[3];
		dot = colors->shade[3];
	}
	else
	{
		border = &colors->shade[5];
		if (widget->prelight)
			border = &colors->spot[1];
		dot.r = 47 / 256.0;
		dot.g = 166 / 256.0;
		dot.b = 0;
	}
	NodokaRGB shadow;
	nodoka_shade (border, &shadow, 0.9);

	cairo_translate (cr, sx, sy);
	cairo_set_line_width (cr, 1);

	if (widget->xthickness > 2 && widget->ythickness > 2)
	{
		cairo_rectangle (cr, 1.5, 1.5, width - 2, height - 2);
		cairo_set_source_rgba (cr, shadow.r, shadow.g, shadow.b, 0.15);
		cairo_stroke (cr);

		/* Draw the rectangle for the checkbox itself */
		cairo_rectangle (cr, 1.5, 1.5, width - 2, height - 2);
	}
	else
	{
		cairo_rectangle (cr, 1.5, 1.5, width - 2, height - 2);
	}


	if (widget->state_type != GTK_STATE_INSENSITIVE)
	{
		const NodokaRGB *bg = &colors->base[0];

		nodoka_set_gradient (cr, bg, 1.1, 0, 14, widget->gradients, FALSE);
		cairo_fill_preserve (cr);
	}

	cairo_set_source_rgba (cr, border->r, border->g, border->b, 0.8);
	cairo_stroke (cr);

	/* shadow */
	if (!(widget->disabled))
	{
		cairo_move_to (cr, 2.5, height - 1);
		cairo_line_to (cr, 2.5, 2.5);
		cairo_line_to (cr, width - 1, 2.5);
		cairo_set_source_rgba (cr, 0.2, 0.2, 0.2, 0.15);
		cairo_stroke (cr);
	}

	cairo_scale (cr, width / 13.0, height / 13.0);
	if (status->draw_bullet)
	{
		if (status->inconsistent)	/* Inconsistent */
		{
			cairo_set_line_width (cr, 2.0);
			cairo_move_to (cr, 3, height * 0.5 + 0.5);
			cairo_line_to (cr, width - 2, height * 0.5 + 0.5);
			cairo_set_source_rgba (cr, dot.r, dot.g, dot.b, trans);
			cairo_stroke (cr);
		}
		else
		{
			cairo_translate (cr, -2, 0);
			cairo_move_to (cr, 4, 8);
			cairo_rel_line_to (cr, 5, 4);
			cairo_rel_curve_to (cr, 1.4, -5, -1, -1, 5.7, -12.5);
			cairo_rel_curve_to (cr, -4, 4, -4, 4, -6.7, 9.3);
			cairo_rel_line_to (cr, -2.3, -2.5);
			cairo_set_source_rgba (cr, dot.r, dot.g, dot.b, trans);
			cairo_fill (cr);
		}
	}

}

void
nodoka_draw_resize_grip (cairo_t * cr,
						 const NodokaColors * colors,
						 const WidgetParameters * widget,
						 const ResizeGripParameters * grip,
						 int x, int y, int width, int height)
{
	NodokaRGB *dark = (NodokaRGB *) & colors->shade[3];
	NodokaRGB hilight;
	nodoka_shade (dark, &hilight, 1.3);
	int lx, ly;
	int bx, ex, by, ey;
	int sx, sy;

	switch (grip->edge)
	{
	case NDK_WINDOW_EDGE_NORTH_WEST:
	{
		by = 0;
		ey = 3;
		sx = x + 10;
		sy = y + 11;
		break;
	}
	case NDK_WINDOW_EDGE_NORTH:
	{
		by = 0;
		ey = 0;
		sx = x + width / 2.0 + 1.5 * 3.5;
		sy = y + 11;
		break;
	}
	case NDK_WINDOW_EDGE_NORTH_EAST:
	{
		by = 0;
		ey = 3;
		sx = x + width;
		sy = y + 11;
		break;
	}
	case NDK_WINDOW_EDGE_WEST:
	{
		by = 0;
		ey = 3;
		sx = x + 10;
		sy = y + height / 2.0 + 1.5 * 3.5;
		break;
	}
	case NDK_WINDOW_EDGE_EAST:
	{
		by = 0;
		ey = 3;
		sx = x + width;
		sy = y + height / 2.0 + 1.5 * 3.5;
		break;
	}
	case NDK_WINDOW_EDGE_SOUTH_WEST:
	{
		by = 0;
		ey = 3;
		sx = x + 10;
		sy = y + height;
		break;
	}
	case NDK_WINDOW_EDGE_SOUTH:
	{
		by = 3;
		ey = 3;
		sx = x + width / 2.0 + 1.5 * 3.5;
		sy = y + height;
		break;
	}
	case NDK_WINDOW_EDGE_SOUTH_EAST:
	{
		by = 0;
		ey = 3;
		sx = x + width;
		sy = y + height;
		break;
	}
	default:
	{
		by = 0;
		ey = 3;
		sx = x + width;
		sy = y + height;
		break;
	}
	}

	cairo_set_line_width (cr, 1);

	for (ly = by; ly <= ey; ly++)	// vertically, four rows of dots
	{
		switch (grip->edge)
		{
		case NDK_WINDOW_EDGE_NORTH_WEST:
		{
			bx = ly;
			ex = 3;
			break;
		}
		case NDK_WINDOW_EDGE_NORTH:
		{
			bx = 0;
			ex = 3;
			break;
		}
		case NDK_WINDOW_EDGE_NORTH_EAST:
		{
			bx = 0;
			ex = 3 - ly;
			break;
		}
		case NDK_WINDOW_EDGE_WEST:
		{
			bx = 3;
			ex = 3;
			break;
		}
		case NDK_WINDOW_EDGE_EAST:
		{
			bx = 0;
			ex = 0;
			break;
		}
		case NDK_WINDOW_EDGE_SOUTH_WEST:
		{
			bx = 3 - ly;
			ex = 3;
			break;
		}
		case NDK_WINDOW_EDGE_SOUTH:
		{
			bx = 0;
			ex = 3;
			break;
		}
		case NDK_WINDOW_EDGE_SOUTH_EAST:
		{
			bx = 0;
			ex = ly;
			break;
		}
		default:
		{
			bx = 0;
			ex = ly;
			break;
		}
		}

		for (lx = bx; lx <= ex; lx++)	// horizontally
		{
			int ny = (3.5 - ly) * 3;
			int nx = lx * 3;

			cairo_set_source_rgb (cr, hilight.r, hilight.g, hilight.b);
			cairo_rectangle (cr, sx - nx - 1, sy - ny - 1, 2, 2);
			cairo_fill (cr);

			cairo_set_source_rgb (cr, dark->r, dark->g, dark->b);
			cairo_rectangle (cr, sx - nx - 1, sy - ny - 1, 1, 1);
			cairo_fill (cr);
		}
	}
}
